/**
 * @file main.c
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * Natural Tiny Basic (NT-Basic) - A tiny BASIC interpreter
 * Version 0.3.0
 * ===============================================================
 * Copyright (c) 2012-2014 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ===============================================================
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ntbasic.h"
#include "progedit.h"

static int load_program(progedit_t *p, const char *filename)
{
  char buf[BUFSIZ];
  int line = 0;
  FILE *fp = fopen(filename, "rb");
  if (fp == NULL) {
    return -1;
  }

  while (fgets(buf, sizeof(buf), fp) != NULL) {
    line++;
    if (progedit_insert(p, buf) != 0) {
      printf("Insert failed.\nline=%d:%s\n", line, buf);
    }
  }

  fclose(fp);
  return line;
}

static int print_program(progedit_t *p)
{
  char buf[BUFSIZ];
  int line = 0;
  while (progedit_fetch(p, line, buf, sizeof(buf)) == 0) {
    line++;
    printf("  %d: [%s]\n", line, buf);
  }
  return line;
}

int main(int argc, char *argv[])
{
  ntbasic_t ntbasic;
  ntbasic_error_t error;
  progedit_t prog;
  int line_files, line_progs;
  char errmsg[BUFSIZ];
  void *extobj = "\"This is a user external object for the test.\"";

  printf("=============================================\n");
  printf(" Natural Tiny Basic (NT-Basic)               \n");
  printf(" The core test program                       \n");
  printf(" ------------------------------------------- \n");
  printf(" Copyright (c) 2012-2014 Shinichiro Nakamura \n");
  printf("=============================================\n");

  /*
   * Check the arguments.
   */
  if (argc != 2) {
    printf("usage: ntbasic <filename>\n");
    return 1;
  }

  /*
   * Load program from a file system.
   */
  progedit_init(&prog);
  line_files = load_program(&prog, argv[1]);
  if (line_files < 0) {
    printf("program read failed.\n");
    return 2;
  }

  printf("[Program list]\n");
  line_progs = print_program(&prog);
  printf("\n");
  printf("[Program information]\n");
  printf("  %d lines readed.\n", line_files);
  printf("  %d lines stored.\n", line_progs);
  printf("  %d bytes.\n", (int)strlen(PROGEDIT_PROGRAM(&prog)));
  printf("\n");

  /*
   * Setup and execute.
   */
  error = ntbasic_setup(&ntbasic, PROGEDIT_PROGRAM(&prog), extobj);
  while (error == NoError) {
    error = ntbasic_execute(&ntbasic);
  }

  /*
   * Check the termination code.
   */
  if (error != EndOfTheProgram) {
    ntbasic_get_error_message(&ntbasic, error, errmsg, sizeof(errmsg));
    printf("Error: %s\n", errmsg);
  }

  return (error == EndOfTheProgram) ? 0 : error;
}

