/**
 * @file progedit.c
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * Natural Tiny Basic (NT-Basic) - A tiny BASIC interpreter
 * Version 0.3.0
 * ===============================================================
 * Copyright (c) 2012-2014 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ===============================================================
 */

#include "ntlibc.h"
#include "progedit.h"

int progedit_init(textedit_t *p)
{
  return textedit_init(p);
}

int progedit_clear(textedit_t *p)
{
  return textedit_clear(p);
}

static int get_line_number(const char *text)
{
  char buf[PROGEDIT_LINESIZE];
  char *p = buf;
  ntlibc_strcpy(buf, text);
  while ((*p != ' ') && (*p != '\0')) {
    p++;
  }
  *p = '\0';
  return ntlibc_atoi(buf);
}

static int cleanup_blank_line(textedit_t *textedit)
{
  int line = 0;
  char buf[PROGEDIT_LINESIZE];
  while (textedit_fetch(textedit, line, buf, sizeof(buf)) == 0) {
    char tmp[PROGEDIT_LINESIZE];
    char *p = tmp;
    ntlibc_strcpy(tmp, buf);
    while ((*p != ' ') && (*p != '\0')) {
      p++;
    }
    if (ntlibc_strlen(p) == 0) {
      textedit_delete(textedit, line);
    } else {
      line++;
    }
  }
  return 0;
}

int progedit_insert(textedit_t *textedit, const char *text)
{
  char tmp[PROGEDIT_LINESIZE];
  char buf[PROGEDIT_LINESIZE];
  int tmplen;
  int line = 0;
  int a, b;
  int i;

  /*
   * Check the text length.
   */
  if (PROGEDIT_LINESIZE <= ntlibc_strlen(text)) {
    return -1;
  }

  /*
   * Copy the master text string to tmp.
   * And fix the termination code to '\n'.
   */
  ntlibc_strcpy(tmp, text);
  tmplen = ntlibc_strlen(tmp);
  for (i = 0; i < tmplen; i++) {
    if ((tmp[i] == '\r') || (tmp[i] == '\n')) {
      tmp[i] = '\0';
    }
  }
  ntlibc_strcat(tmp, "\n");

  /*
   * Ignore the blank line.
   */
  if (ntlibc_strlen(tmp) <= 1) {
    return 0;
  }

  /*
   * Check the line number.
   */
  a = get_line_number(tmp);
  if (a < 0) {
    return -1;
  }

  /*
   * Insert the target text string.
   */
  while (textedit_fetch(textedit, line, buf, sizeof(buf)) == 0) {
    b = get_line_number(buf);
    if (a < b) {
      if (textedit_insert(textedit, line, tmp) != 0) {
        return -1;
      }
      cleanup_blank_line(textedit);
      return 0;
    } else if (a == b) {
      if (textedit_delete(textedit, line) != 0) {
        return -1;
      }
      if (textedit_insert(textedit, line, tmp) != 0) {
        return -1;
      }
      cleanup_blank_line(textedit);
      return 0;
    }
    line++;
  }
  if (textedit_insert(textedit, line, tmp) != 0) {
    return -1;
  }
  cleanup_blank_line(textedit);
  return 0;
}

int progedit_fetch(textedit_t *p, const int line_number, char *text, const int maxsize)
{
  return textedit_fetch(p, line_number, text, maxsize);
}

